struct lightSourceParameters {
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	vec4 position;
	vec3 spotDirection;
	float spotExponent;
	float spotCutoff;
	float spotCosCutoff;
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;	
};
void init(in vec3 ecPosition3, in vec3 eye, in vec3 normal, inout vec4 diffuse, inout vec4 spec);
uniform bool sun;
uniform sampler2D tex0;
uniform sampler2D bumpMap;
uniform int texCount;
uniform lightSourceParameters light[1];
varying vec3 normal;
varying vec3 worldPosition;
uniform vec4 viewer;
uniform bool ambient;
uniform bool bump;
uniform bool isBullet;

varying vec3 tang;
varying vec3 binormale;

void PointLight(in int i, in vec3 eye, in vec3 ecPosition3, in vec3 normal, inout vec4 ambient, inout vec4 diffuse, inout vec4 specular, in vec3 lPos);

void main(void)
{
   vec4 amb = vec4(0.0);
   vec4 dif = vec4(0.0);
   vec4 spec = vec4(0.0);
   vec3 theEye = vec3(viewer) - worldPosition;
   
   vec3 normale = normalize(normal);
   
   	if (sun) {
   		theEye = -theEye;
		normale = -normale;
	}
	
	vec3 color = vec3(0.0, 0.0, 0.0);
	vec3 lightPos = light[0].position.xyz - worldPosition;
	theEye = normalize(theEye);
	vec2 offset = vec2(0.0);
	
	if (bump) {
		vec3 tTang = normalize(-tang);
		vec3 bBinorm = normalize(binormale);
		theEye = vec3(dot(tTang, theEye), dot(bBinorm, theEye), dot(normale, theEye));
		lightPos = vec3(dot(tTang, lightPos), dot(bBinorm, lightPos), dot(normale, lightPos));
		
		//for (int i = 0; i < 6; ++i) {
			//vec4 normalTemp = texture2D(bumpMap, gl_TexCoord[0].st + offset);
			//offset -= theEye.xy * ((normalTemp.a/100.0 + 0.001) * normalTemp.z) / dot(theEye, vec3(normalTemp.x, normalTemp.y, abs(min(normalTemp.z, theEye.z)-theEye.z)*normalTemp.z));
			//offset += theEye.xy * (normalTemp.a/100.0 + 0.001) / theEye.z;
			//offset = clamp(offset,-0.08,0.08);
		//}

		//normale = normalize(vec3(texture2D(bumpMap, gl_TexCoord[0].st + offset)));

  		vec4 normalTemp = texture2D(bumpMap, vec2(gl_TexCoord[0].s, -gl_TexCoord[0].t) + offset);
  		for (int i = 0; i < 6; ++i) {
   			offset = theEye.xy * (normalTemp.a/30.0 + 0.001);
 			normalTemp = (normalTemp + texture2D(bumpMap, vec2(gl_TexCoord[0].s, -gl_TexCoord[0].t) + offset)) * 0.5;
  		}

   		normale = normalize((normalTemp.xyz * 2.0) - 1.0);
	}
	
	vec4 temp = texture2D(tex0, vec2(gl_TexCoord[0].s, gl_TexCoord[0].t) + offset);
	
	if (ambient) {
		amb += 0.05;
		gl_FragData[2] = amb * temp;
		gl_FragData[2].a = 0.0;
	} else {
		init(worldPosition, theEye, normale, dif, spec);
		gl_FragData[2] = 6.0 * spec;
		gl_FragData[2].a = 0.0;
	}
    if (isBullet) {
		gl_FragData[1] = vec4(0, 0.8, 0.3, 1.0);
		gl_FragData[0] = gl_FragData[1];
	} else {
		PointLight(0, theEye, worldPosition, normale, amb, dif, spec, lightPos);
   
		color += vec3(dif * temp);
		vec3 colorT = color + vec3(spec);
		float depth = (gl_FragCoord.z/gl_FragCoord.w);
		depth = depth / 5000.0;
		vec4 tempRes = vec4(color + vec3(spec) / 100.0, depth);
		gl_FragData[0] = tempRes;
		gl_FragData[1] = vec4(step(3.0, colorT.r + colorT.g + colorT.b) * vec3(tempRes), depth);
	}
}

void PointLight(in int i, in vec3 eye, in vec3 ecPosition3, in vec3 normal, inout vec4 ambienti, inout vec4 diffusi, inout vec4 speculari, in vec3 lPos) {
   float nDotVP;
   float nDotHV;
   float pf;
   float attenuation;
   float d;
   vec3 VP;
   vec3 halfVector;
   
   VP = lPos;
   d = length(VP);
   VP = normalize(VP);
   
   attenuation = 1.0/(light[i].constantAttenuation +
                      light[i].linearAttenuation * d +
                      light[i].quadraticAttenuation * d * d);
     
   halfVector = normalize(VP + eye);
   
   nDotVP = max(0.0, dot(normal, VP));
   nDotHV = max(0.0, dot(normal, halfVector));
     
   if(nDotVP == 0.0) {
      pf = 0.0;
   } else {
      pf = pow(nDotHV, gl_FrontMaterial.shininess);
   }
   
   speculari = light[i].specular * pf * attenuation;
   ambienti += light[i].ambient * attenuation;
   diffusi += light[i].diffuse * nDotVP;// * 2.0 * attenuation;
}