uniform float maxD;
uniform float linearAttenuation;
uniform float quadraticAttenuation;
uniform sampler2D difMapS;
uniform sampler2D specMapS;

void getTextureLightingDiffuse(in sampler2D difMap, in vec3 normal, inout vec4 diffuse, in vec3 pos, in float q, in float d);
void getTextureLightingSpecular(in sampler2D specMap, in vec3 eye, in vec3 normal, inout vec4 spec, in vec3 pos, in float q, in float d);

void getTextureLightingDiffuse(in sampler2D difMap, in vec3 normal, inout vec4 diffuse, in vec3 pos, in float q, in float d) {
	float t = dot(pos, normal) / dot(normal, normal);
	
	float landa = t + sqrt(t*t + 1.0 - q);

   	vec3 temp = pos + normal * landa;
  	vec3 lookup = vec3(0.0, (temp.y + 1.0) / 2.0, 0.0);
  	temp.y = 0.0;
   	lookup.xz = normalize(temp).xz;
   	lookup.x = 0.5 + sign(lookup.z) * (lookup.x / 4.0 - 0.25);

	diffuse += texture2D(difMap, lookup.xy) / d;
}

void getTextureLightingSpecular(in sampler2D specMap, in vec3 eye, in vec3 normal, inout vec4 spec, in vec3 pos, in float q, in float d) {

	vec3 reflected = reflect(eye, normal);
	
	getTextureLightingDiffuse(specMap, reflected, spec, pos, q, d);
}

void init(in vec3 ecPosition3, in vec3 eye, in vec3 normal, inout vec4 diffuse, inout vec4 spec) {
	vec3 pos = 0.0;
	float q = dot(pos, pos);
	float d = 1.0 - length(pos);
	d = 1.0 + linearAttenuation * d + quadraticAttenuation * d * d;

	getTextureLightingDiffuse(difMapS, normal, diffuse, pos, q, d);
	getTextureLightingSpecular(specMapS, eye, normal, spec, pos, q, d);
}